@@ -1,4 +1,4 @@
-(* $Id: checkItem.mli,v 1.9 2007/01/19 01:53:16 ddr Exp $ *)
+(* $Id: checkItem.mli,v 1.12 2007/09/05 13:19:25 ddr Exp $ *)
 (* Copyright (c) 2006-2007 INRIA *)
 
 open Gwdb;
@@ -23,4 +23,5 @@ value family :
   base -> (base_error -> unit) -> (base_warning -> unit) -> Def.ifam ->
     family -> unit;
 
-value sort_children : base -> array Adef.iper -> option (array Adef.iper);
+value sort_children :
+  base -> array Adef.iper -> option (array Adef.iper * array Adef.iper);
