@@ -1,8 +1,8 @@
 #!/bin/sh
 #cd (*
-exec ocaml camlp4r.cma $0
+exec ocaml -I +camlp5 camlp5r.cma $0
 *) ".";
-(* $Id: mk_missing_i18n.sh,v 5.3 2006/10/31 14:12:03 ddr Exp $ *)
+(* $Id: mk_missing_i18n.sh,v 5.4 2007/07/28 10:13:42 ddr Exp $ *)
 
 open Printf;
 
