@@ -1,5 +1,5 @@
-(* camlp4r ./pa_html.cmo *)
-(* $Id: some.ml,v 5.40 2007/02/24 19:46:21 ddr Exp $ *)
+(* camlp5r ./pa_html.cmo *)
+(* $Id: some.ml,v 5.44 2007/09/12 09:58:44 ddr Exp $ *)
 (* Copyright (c) 1998-2007 INRIA *)
 
 open Config;
@@ -96,7 +96,23 @@ value print_elem conf base is_surname (p, xl) =
     xl
 ;
 
-value first_name_print_list conf base x1 xl liste =
+value first_char s =
+  if Mutil.utf_8_db.val then
+    let len = Name.nbc s.[0] in
+    if len < String.length s then String.sub s 0 len
+    else s
+  else String.sub s (initial s) 1
+;
+
+value name_unaccent s =
+  copy 0 0 where rec copy i len =
+    if i = String.length s then Buff.get len
+    else
+      let (t, j) = Name.unaccent_utf_8 False s i in
+      copy j (Buff.mstore len t)
+;
+
+value first_name_print_list conf base x1 xl liste = do {
   let liste =
     let l =
       List.sort
@@ -131,14 +147,21 @@ value first_name_print_list conf base x1 xl liste =
              (if first then "" else ", ") (commd conf) (code_varenv x) x)
         (StrSet.elements xl)
   in
-  do {
-    header conf title;
-    print_link_to_welcome conf True;
-    print_alphab_list conf (fun (p, _) -> String.sub p (initial p) 1)
-      (print_elem conf base True) liste;
-    trailer conf;
-  }
-;
+  header conf title;
+  print_link_to_welcome conf True;
+  let list =
+    List.map
+      (fun (sn, ipl) ->
+         let txt = Util.surname_end base sn ^ Util.surname_begin base sn in
+         let ord = name_unaccent txt in
+         (ord, txt, ipl))
+      liste
+  in
+  let list = List.sort compare list in
+  print_alphab_list conf (fun (ord, txt, _) -> first_char ord)
+    (fun (_, txt, ipl) -> print_elem conf base True (txt, ipl)) list;
+  trailer conf;
+};
 
 value select_first_name conf base n list =
   let title _ =
@@ -229,8 +252,8 @@ value first_name_print conf base x =
   | _ -> select_first_name conf base x list ]
 ;
 
-value has_children_with_that_name base des name =
-  List.exists (fun ip -> p_surname base (poi base ip) = name)
+value has_children_with_that_name conf base des name =
+  List.exists (fun ip -> p_surname base (pget conf base ip) = name)
     (Array.to_list (get_children des))
 ;
 
@@ -284,14 +307,14 @@ type branch_head 'a = { bh_ancestor : 'a; bh_well_named_ancestors : list 'a };
 value print_branch conf base psn name =
   let unsel_list = unselected_bullets conf in
   loop where rec loop is_first_level p = do {
-    let u = uget conf base (get_key_index p) in
+    let u = pget conf base (get_key_index p) in
     let family_list =
       List.map
         (fun ifam ->
            let fam = foi base ifam in
            let c = spouse (get_key_index p) fam in
            let c = pget conf base c in
-           let down = has_children_with_that_name base fam name in
+           let down = has_children_with_that_name conf base fam name in
            let down =
              if get_sex p = Female && p_surname base c = name then False
              else down
@@ -486,14 +509,6 @@ value print_one_surname_by_branch conf base (bhl, str) = do {
   trailer conf;
 };
 
-value name_unaccent s =
-  copy 0 0 where rec copy i len =
-    if i = String.length s then Buff.get len
-    else
-      let (t, j) = Name.unaccent_utf_8 False s i in
-      copy j (Buff.mstore len t)
-;
-
 value print_several_possible_surnames x conf base (bhl, homonymes) = do {
   let fx = x in
   let x =
@@ -537,14 +552,6 @@ value print_several_possible_surnames x conf base (bhl, homonymes) = do {
   trailer conf;
 };
 
-value first_char s =
-  if Mutil.utf_8_db.val then
-    let len = Name.nbc s.[0] in
-    if len < String.length s then String.sub s 0 len
-    else s
-  else String.sub s (initial s) 1
-;
-
 value print_family_alphabetic x conf base liste =
   let homonymes =
     let list =
@@ -618,8 +625,7 @@ value select_ancestors conf base name_inj ipl =
   List.fold_left
     (fun bhl ip ->
        let p = pget conf base ip in
-       let a = aget conf base ip in
-       match get_parents a with
+       match get_parents p with
        [ Some ifam ->
            let fam = foi base ifam in
            let ifath = get_father fam in
@@ -722,9 +728,9 @@ value surname_print conf base not_found_fun x =
       let bhl =
         List.map
           (fun bh ->
-             {bh_ancestor = poi base bh.bh_ancestor;
+             {bh_ancestor = pget conf base bh.bh_ancestor;
               bh_well_named_ancestors =
-                List.map (poi base) bh.bh_well_named_ancestors})
+                List.map (pget conf base) bh.bh_well_named_ancestors})
           bhl
       in
       match (bhl, strl) with
